@@ -1,4 +1,9 @@
 
+process_has_input_connection <- function(self, private) {
+  "!DEBUG process_has_input_connection `private$get_short_name()`"
+  !is.null(private$stdin_pipe)
+}
+
 process_has_output_connection <- function(self, private) {
   "!DEBUG process_has_output_connection `private$get_short_name()`"
   !is.null(private$stdout_pipe)
@@ -9,6 +14,18 @@ process_has_error_connection <- function(self, private) {
   !is.null(private$stderr_pipe)
 }
 
+process_has_poll_connection <- function(self, private) {
+  "!DEBUG process_has_error_connection `private$get_short_name()`"
+  !is.null(private$poll_pipe)
+}
+
+process_get_input_connection <- function(self, private) {
+  "!DEBUG process_get_input_connection `private$get_short_name()`"
+  if (!self$has_input_connection())
+    stop("stdin is not a pipe.")
+  private$stdin_pipe
+}
+
 process_get_output_connection <- function(self, private) {
   "!DEBUG process_get_output_connection `private$get_short_name()`"
   if (!self$has_output_connection())
@@ -23,6 +40,12 @@ process_get_error_connection <- function(self, private) {
   private$stderr_pipe
 }
 
+process_get_poll_connection <- function(self, private) {
+  "!DEBUG process_get_poll_connection `private$get_short_name()`"
+  if (!self$has_poll_connection()) stop("No poll connection")
+  private$poll_pipe
+}
+
 process_read_output <- function(self, private, n) {
   "!DEBUG process_read_output `private$get_short_name()`"
   con <- process_get_output_connection(self, private)
@@ -94,6 +117,20 @@ process_read_all_error_lines <- function(self, private) {
   results
 }
 
+process_write_input <- function(self, private, str, sep) {
+  "!DEBUG process_write_input `private$get_short_name()`"
+  con <- process_get_input_connection(self, private)
+  if (is.character(str)) {
+    pstr <- paste(str, collapse = sep)
+    str <- iconv(pstr, "", private$encoding, toRaw = TRUE)[[1]]
+  }
+  invisible(.Call(c_processx_connection_write_bytes, con, str))
+}
+
+process_get_input_file <- function(self, private) {
+  private$stdin
+}
+
 process_get_output_file <- function(self, private) {
   private$stdout
 }
