@@ -6,6 +6,11 @@
 extern "C" {
 #endif
 
+#ifdef __INTEL_COMPILER
+#define _BSD_SOURCE 1
+#define _POSIX_C_SOURCE  200809L
+#endif
+
 #ifdef _WIN32
 #include <windows.h>
 #else
@@ -31,27 +36,33 @@ extern "C" {
 
 /* API from R */
 
-SEXP processx_exec(SEXP command, SEXP args, SEXP std_out, SEXP std_err,
+SEXP processx_exec(SEXP command, SEXP args,
+		   SEXP std_in,  SEXP std_out, SEXP std_err,
+		   SEXP connections, SEXP env,
 		   SEXP windows_verbatim_args,
 		   SEXP windows_hide_window, SEXP private_, SEXP cleanup,
-		   SEXP encoding);
+		   SEXP wd, SEXP encoding, SEXP tree_id);
 SEXP processx_wait(SEXP status, SEXP timeout);
 SEXP processx_is_alive(SEXP status);
 SEXP processx_get_exit_status(SEXP status);
 SEXP processx_signal(SEXP status, SEXP signal);
+SEXP processx_interrupt(SEXP status);
 SEXP processx_kill(SEXP status, SEXP grace);
 SEXP processx_get_pid(SEXP status);
+SEXP processx_create_time(SEXP r_pid);
 
-SEXP processx_poll(SEXP statuses, SEXP ms);
+SEXP processx_poll(SEXP statuses, SEXP conn, SEXP ms);
 
 SEXP processx__process_exists(SEXP pid);
-SEXP processx__disconnect_process_handle(SEXP status);
+SEXP processx__proc_start_time(SEXP status);
 
 SEXP processx_is_named_pipe_open(SEXP pipe_ext);
 SEXP processx_close_named_pipe(SEXP pipe_ext);
 SEXP processx_create_named_pipe(SEXP name, SEXP mode);
 SEXP processx_write_named_pipe(SEXP pipe_ext, SEXP text);
 
+SEXP processx_disable_crash_dialog();
+
 /* Common declarations */
 
 /* Interruption interval in ms */
@@ -78,6 +89,7 @@ SEXP processx_write_named_pipe(SEXP pipe_ext, SEXP text);
 typedef struct {
   int windows_verbatim_args;
   int windows_hide;
+  const char *wd;
 } processx_options_t;
 
 #ifdef __cplusplus
